@@ -1,7 +1,7 @@
 from datetime import timedelta
 
 from django.contrib.auth import get_user_model
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.utils import timezone
 
 from oauthlib.oauth2 import (InvalidClientIdError, MissingClientIdError,
@@ -20,26 +20,27 @@ User = get_user_model()
 
 
 class BaseTest(TestCaseUtils):
-    def setUp(self):
-        self.test_user = User.objects.create_user('test_user', 'test_user@example.com', '1234')
-        self.dev_user = User.objects.create_user('dev_user', 'dev_user@example.com', '1234')
-        self.application = Application(
+    @classmethod
+    def setUpTestData(cls):
+        cls.test_user = User.objects.create_user('test_user', 'test_user@example.com', '1234')
+        cls.dev_user = User.objects.create_user('dev_user', 'dev_user@example.com', '1234')
+        cls.application = Application(
             name='Test Application',
             redirect_uris='http://localhost http://example.com',
-            user=self.dev_user,
+            user=cls.dev_user,
             client_type=Application.CLIENT_CONFIDENTIAL,
             authorization_grant_type=Application.GRANT_AUTHORIZATION_CODE,
         )
-        self.application.save()
+        cls.application.save()
 
-        self.application_public = Application(
+        cls.application_public = Application(
             name='Test Application (Public)',
             redirect_uris='http://localhost http://example.com',
-            user=self.dev_user,
+            user=cls.dev_user,
             client_type=Application.CLIENT_PUBLIC,
             authorization_grant_type=Application.GRANT_AUTHORIZATION_CODE,
         )
-        self.application_public.save()
+        cls.application_public.save()
 
 
 class TestAuthorizationCode(BaseTest):
@@ -118,7 +119,7 @@ class TestAuthorizationCode(BaseTest):
         response = self.client.get(reverse('oauth_api:authorize'), data=query_string)
 
         self.assertEqual(response.status_code, status.HTTP_302_FOUND)
-        self.assertIn('error=unauthorized_client', response['Location'])
+        self.assertIn('error=unsupported_response_type', response['Location'])
 
     def test_missing_response_type(self):
         """
@@ -253,7 +254,8 @@ class TestAuthorizationCode(BaseTest):
         response = self.client.post(reverse('oauth_api:authorize'), data=form_data)
 
         self.assertEqual(response.status_code, status.HTTP_302_FOUND)
-        self.assertIn('http://localhost?error=unauthorized_client', response['Location'])
+        self.assertIn('error=unsupported_response_type', response['Location'])
+        self.assertIn('state=random_state_string', response['Location'])
 
     def test_authorization_code_post_invalid_redirect_uri(self):
         """
@@ -317,7 +319,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
 
         response = self.client.post(reverse('oauth_api:token'), token_request)
 
-        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
 
     def test_invalid_auth_code(self):
         """
@@ -336,7 +338,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
 
         response = self.client.post(reverse('oauth_api:token'), token_request)
 
-        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
 
     def test_invalid_grant_type(self):
         """
@@ -377,7 +379,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
                                                                        self.application.client_secret))
 
         response = self.client.post(reverse('oauth_api:token'), token_request)
-        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
 
     def test_refresh_token(self):
         """
@@ -425,7 +427,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
 
         # Refresh tokens cannot be used twice
         response = self.client.post(reverse('oauth_api:token'), token_request)
-        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
         self.assertTrue('invalid_grant' in response.data.values())
 
     def test_refresh_token_override_authorization(self):
@@ -557,7 +559,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
         self.assertEqual(response.status_code, status.HTTP_200_OK)
 
         response = self.client.post(reverse('oauth_api:token'), token_request)
-        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
 
     def test_refresh_token_expired(self):
         """
@@ -590,7 +592,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
         RefreshToken.objects.update(expires=timezone.now())
 
         response = self.client.post(reverse('oauth_api:token'), token_request)
-        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
 
     def test_refresh_token_not_expired(self):
         """
@@ -636,11 +638,9 @@ class TestAuthorizationCodeTokenView(BaseTest):
             'grant_type': 'authorization_code',
             'code': authorization_code,
             'redirect_uri': 'http://localhost',
+            'client_id': self.application_public.client_id,
         }
 
-        self.client.credentials(HTTP_AUTHORIZATION=self.get_basic_auth(self.application_public.client_id,
-                                                                       self.application_public.client_secret))
-
         response = self.client.post(reverse('oauth_api:token'), token_request)
 
         self.assertEqual(response.status_code, status.HTTP_200_OK)
@@ -648,6 +648,24 @@ class TestAuthorizationCodeTokenView(BaseTest):
         self.assertEqual(response.data['scope'], 'read write')
         self.assertEqual(response.data['expires_in'], oauth_api_settings.ACCESS_TOKEN_EXPIRATION)
 
+    def test_public_with_invalid_redirect_uri(self):
+        """
+        Test for requesting access token using client_type 'public' and providing
+        invalid redirect_uri
+        """
+        self.client.login(username='test_user', password='1234')
+        authorization_code = self.get_authorization_code(self.application_public.client_id)
+
+        token_request = {
+            'grant_type': 'authorization_code',
+            'code': authorization_code,
+            'redirect_uri': '/../',
+            'client_id': self.application_public.client_id,
+        }
+
+        response = self.client.post(reverse('oauth_api:token'), token_request)
+        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
+
 
 class TestAuthorizationCodeResourceAccess(BaseTest):
     def test_access_allowed(self):
